/*

The NetUsage Project
http://netusage.mozdev.org/

Copyright (C) 2006, NetUsage Project
This extension can be freely distributed under
the terms of the Mozilla Public Licence 1.1
See the licence at http://www.mozilla.org/MPL/

---------
Net Usage Item

Function Library

*/

const __USAGELIB_JS__ = 1;

// for isps.js
const PREF_TYPE_INT = 1;
const PREF_TYPE_STRING = 2;
const PREF_TYPE_BOOL = 4;
const PREF_FLAG_REQUIRED = 8;
const PREF_FLAG_GENERAL = 16;
const PREF_FLAG_LIST_EDITABLE = 32;


// load user's password
function nuLoadPass(ahost) {
	var passManInt = Components.classes['@mozilla.org/passwordmanager;1'].getService().QueryInterface(Components.interfaces.nsIPasswordManagerInternal);
	var fhost = {value: ''};
	var fuser = {value: ''};
	var fpassword = {value: ''};

	try {
		passManInt.findPasswordEntry(ahost, '', '', fhost, fuser, fpassword);
	} catch(e) {
		dump(e+'\n'); 
		return false;
	}

	return [fhost.value, fuser.value, fpassword.value];
}

// save user's password
function nuSavePass(ahost, ausername, apassword) {
	var passMan = Components.classes['@mozilla.org/passwordmanager;1'].getService().QueryInterface(Components.interfaces.nsIPasswordManager);

	while (todel = nuLoadPass(ahost)) {
		passMan.removeUser(ahost, todel[1]);
	}
	if (ausername != null) {
		passMan.addUser(ahost, ausername, apassword);
	}
}

// my new foolproof encode base64 for username/password auth :)
function nuEncodeBase64(str) {

	var b64 = [
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',   //  0 to  7
	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',   //  8 to 15
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',   // 16 to 23
	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',   // 24 to 31
	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',   // 32 to 39
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',   // 40 to 47
	'w', 'x', 'y', 'z', '0', '1', '2', '3',   // 48 to 55
	'4', '5', '6', '7', '8', '9', '+', '/' ]; // 56 to 63

	var tfb = 0, res = '';
	
	// do a 3-to-4
	for (var i = 0; i < str.length; i += 3) {
		tfb = 0;
		tfb |= str.charCodeAt(i) << 16;
		if (i+1 < str.length) {
			tfb |= str.charCodeAt(i+1) << 8;
			if (i+2 < str.length)
				tfb |= str.charCodeAt(i+2);
		}
	
		res += b64[tfb >>> 18 & 63] 
			+ b64[tfb >>> 12 & 63] 
			+ b64[tfb >>> 6 & 63]
			+ b64[tfb & 63];
	}

	// take care of padding
	if (str.length % 3 == 1) {
		res = res.substr(0, 4*parseInt(str.length/3)+2)+'==';
	} else if (str.length % 3 == 2) {
		res = res.substr(0, 4*parseInt(str.length/3)+3)+'=';
	}

	return res;
}


function ndump(s) {
	dump("NETUSAGE >>> " + s + "\n");
}

function ndumpobj(objname, o) {
	dump('NETUSAGE >>> EEEEEEEEE '+objname+' = {\n');
	for (var i in o) {
		dump(i+': '+o[i]+'\n');
	}
	dump('} EEEEEEEEEE\n');
}

function ndumpobjstr(objname, o) {
	var s = 'NETUSAGE >>> EEEEEEEEE '+objname+' = {\n';
	for (var i in o) {
		s += i+': '+o[i]+'\n';
	}
	s += '} EEEEEEEEEE\n';
	return s;
}

function nuLoadPrefList(branch, vars) {
	try {
		var prefSvc = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var pref = prefSvc.getBranch(branch);
		var childs = pref.getChildList("", { });
		for (var i = 0; i < childs.length; ++i) {
			try {
			ndump('gp '+childs[i]);
			switch(pref.getPrefType(childs[i])) {
				case Components.interfaces.nsIPrefBranch.PREF_STRING:
					vars[childs[i]] = pref.getCharPref(childs[i]);
					break;
				case Components.interfaces.nsIPrefBranch.PREF_INT:
					vars[childs[i]] = pref.getIntPref(childs[i]);
					break;
				case Components.interfaces.nsIPrefBranch.PREF_BOOL:
					vars[childs[i]] = pref.getBoolPref(childs[i]);
					break;
			}
			} catch(e) { ndump(e); }
		}
	} catch(e) { ndump(e); }
}

function nuVarPipe(vs, ivalue, vars, defaultEscape, defaultVarOverwrite, mapvars, isp, internalfunclist, internalfunc) {
	var a = vs.split('|');
	var v = a[0];
	
	// initial value?
	if (ivalue != null) {
		// if it's from XML, reverse the array as the pipe goes right to left
		value = ivalue;
		a = a.reverse();
	}
	else if (typeof(vars[v]) != 'undefined')
		value = vars[v];
	else
		return '{'+vs+'}';
	
	if (mapvars || typeof(vars[a[a.length-1]]) == 'undefined') {
		// run the filter pipeline
		for (var i in a) {
			if ((i == 0 && ivalue == null) || a[i] == 'NOESC')
				continue;
			if (internalfunclist.indexOf(a[i]) != -1) // internal function
				value = internalfunc(a[i], value, isp);
			else if (typeof(isp[a[i]]) == 'function') // custom function
				value = isp[a[i]](value);
			else if (mapvars && (defaultVarOverwrite || typeof(vars[a[i]]) == 'undefined')) // copy variable
				vars[a[i]] = value;
			// if defaultVarOverwrite == false then we can preserve values already present (ie those set in preferences)
		}
	} else {
		// if we are not mapping vars and last term of pipeline is an existing variable
		// use it as our var
		value = vars[a[a.length-1]];
	}
	
	if (!defaultEscape || a.indexOf('NOESC') != -1)
		return value; 
	else
		return encodeURIComponent(value);
}

function nuVarExpand(s, vars, defaultEscape, defaultVarOverwrite, mapvars, isp, internalfunclist, internalfunc) {
	return s.replace(/\{[A-Za-z0-9:_\-\|]+\}/g, function(vs) {
		return nuVarPipe(vs.substr(1, vs.length-2), null, vars, defaultEscape, defaultVarOverwrite, mapvars, isp, internalfunclist, internalfunc);
	} );
}

function nuEscapeRegexp(s) {
	// escapes \ / * + ? | { } [ ] ( ) ^ $ . #
	return s.replace(/([\\\/\*\+\?\|\{\}\[\]\(\)\^\$\.\#]{1})/g, '\\$1');
}

function nuGetVersion() {
	var extMan = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
	var ext = extMan.getItemForID("{DA1B0AB5-7DD3-4066-BC2A-64AABBDD0A8B}");
	ext.QueryInterface(Components.interfaces.nsIUpdateItem);
	return ext.version;
}

function nuGetISPList(ct) {
	var scriptinc = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService().QueryInterface(Components.interfaces.mozIJSSubScriptLoader);
	try {
		if (ct == 'au')
			scriptinc.loadSubScript("chrome://netusageitem/content/isps.js");
		else
			scriptinc.loadSubScript("chrome://netusageisp"+ct+"/content/isps_"+ct+".js");
	} catch(e) {
		return { };
	}
	return eval('nu_isps_' + ct);
}

function nuGetISPInterface(ispid) {
	var ct, obj;
	if (ispid.indexOf('.') != -1)
		ct = ispid.substring(0, ispid.indexOf('.'));
	else
		ct = 'au';
	
	obj = nuGetISPList(ct);

	if (obj[ispid] != undefined) {
		obj[ispid].id = ispid;
		return obj[ispid];
	} else
		return null;
}
